/* ==========================================
 ** ******************************************
 ** catmail.c Version  2.50
 ** -----------------------
 ** Erstellt Mails fr Cat.
 ** Anwendung: Automatisches Erstellen von
 **            Mails aus Scripten oder 
 **            Programmen
 ** Status   : Public Domain
 ** Compiler : PureC 1.1
 ** Rechte   : Hoffentlich bald weniger
 **            Gib Nazis keine Chance!
 ** Autor(en): Wolfgang Wander (wwc)
 **          : ...
 ** ChangeLog: 20.11.1993 Erstellung (wwc)
 **            24.12.1993 ;-) Prfen auf 
 **             vorhandene Messages (wwc)
 ** ******************************************
 * ========================================== */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include <ext.h>
#include <tos.h>


/* -----------------------------------------
 * Eine kleine Funktionsbeschreibung
 * ----------------------------------------- */

static void
syntax(void)
{
	fprintf(stderr,
		"catmail (2.50):\n\r"
		"   Aufruf:     catmail [Optionen]\n\r"
		"       Optionen:\n\r"
		"     -a Gruppenname   Sendet eine ffentliche Mail\n\r"
		"                      in eine Mausgruppe, sollte wohl\n\r"
		"                      besser nicht automatisiert werden.\n\r"
		"     Adresse          Sendet eine persnliche Mail an\n\r"
		"                      die angegebene Adresse. Adresse\n\r"
		"                      darf entweder keine Freizeichen\n\r"
		"                      enthalten (durch _ ersetzen) oder\n\r"
		"                      muss in entsprechenden Shells ge-\n\r"
		"                      quotet werden, falls Leerzeichen\n\r"
		"                      auftauchen.\n\r"
		"     -i Textfile      Liest ein Textfile als den Message\n\r"
		"                      Text ein. Ansonsten wird von stdin\n\r"
		"                      gelesen. (Klar - der Cat Editor ist\n\r"
		"                      besser ,-) aber hier kann man z.B. durch \n\r"
		"                      PGP pipen)\n\r"
		"     -s Betreff       Setzt den Betreff. Dieser Schalter\n\r"
		"                      muss angegeben werden.\n\r"
		"---> Taste <---");
	getch();
	fprintf(stderr, "\n\r"
		" Die Environmentvariable CATMAIL mu auf die Datei CAT.INF\n\r"
		" zeigen oder catmail mu im Verzeichnis von CAT aufgerufen\n\r"
		" werden.     Status: Public Domain\n\r"
		"---> Taste <---");
	getch();
	fprintf(stderr,"\n\r");
	exit(1);
}

/* -----------------------------
 * einige Datentypen
 * ----------------------------- */ 

typedef char Path[512];

typedef struct {  /* Interne zu fllende Struktur von catmail */
	Path inf_path;
	Path message_path;
	int  msg_number; /* erste freie Msgs-Nummer */
} CatInfo;

typedef enum {    /* Cat Messagetypen */
	comment, 
	answer, 
	mail, 
	private, 
	groupcomment, 
	copyOwn,
	copyOther,
	forward,
	persanswer
} MessageType;

typedef enum {     /* Cat Distribution */
	dNone,
	dLokal,
	dMausNet,
	dNet
} tDistribution;

typedef enum {     /* Der Boolean */
	false,
	true
} Boolean;

typedef struct {   /* Der Header von msginfo.dat */
	unsigned long cat_magic;
	unsigned      version;
	unsigned      version_magic;
} FileHeaderType;

typedef struct {  /* Ein Record von msginfo.dat */
	MessageType   msg_type;
	int           number;
	int           copy_count;
	int           ref_num;
	Boolean       send_msg;
	Boolean       to_delete;
	tDistribution dist;
	unsigned long size;
	Boolean       no_ref_line;
	unsigned      c_date;
	unsigned      c_time;
	tDistribution org_dist;
	unsigned long reserved1;
	unsigned long reserved2;
} ShortInfo;

/* ---------------------------------
 * Konstanten
 * --------------------------------- */

const FileHeaderType magic = { /* So sollte der Header aussehen! */
	'CATM', 2, 0x0160
};

#define cat_max_msgs  999      /* maximale anzahl von outg. msgs */
const int next_read    = 0xffffu;

/* ==================================
 * Funktionen zum Lesen und Schreiben
 * der msginfo.dat
 * ==================================

/* ---------------------------
 * prft, ob der Header gltig
 * ist
 * --------------------------- */

static Boolean 
dat__check( FILE *f ) 
{
	FileHeaderType to_check;
	
	if( ftell(f) )
		rewind(f);
	
	return  
	 ( 1 != fread( &to_check, sizeof to_check, 1U, f) 
	   || memcmp( &to_check, &magic, sizeof to_check) ) ? false : true;
}

/* -------------------------------
 * bestimmt die Anzahl der Records
 * in msginfo.dat. -1: Fehler
 * ------------------------------- */
 
static int
dat__length( FILE *f)
{
	long   len = filelength( fileno(f));
	ldiv_t div;
	
	len -= sizeof(FileHeaderType);
	div  = ldiv( len, (long)sizeof( ShortInfo) );
	
	return div.rem ? -1 : (int)div.quot;
}

/* -----------------------------------
 * Liest einen Eintrag aus der Msginfo
 * number: Nummer des Records im 
 *         msginfo
 *         nummer = next_read: liest nchsten
 *                      Eintrag 
 *                      (24.12.93)
 * ----------------------------------- */

static Boolean
dat__read( FILE *f, ShortInfo *info, unsigned number)
{
	long fpos;

	if( number != next_read ) /* -- wwc 24.12.93 */
		fpos =  (long)( sizeof(FileHeaderType) + 
	                number * sizeof( ShortInfo) );

	return  ( ( ftell(f) != fpos && fseek( f, fpos, SEEK_SET) ) 
	          || 1 != fread( info, sizeof( ShortInfo), 1, f) )
	        ? false : true;
}

/* -----------------------------------
 * Sucht nach der ersten freien Msg-
 * nummer in der Datenbank. Wenn keine
 * mehr vorhanden ist - > return -1
 * wwc 24.12.93
 * ----------------------------------- */

static int
dat__firstfree( FILE *f)
{
	ShortInfo info;
	int num, i;
	static Boolean freerec[cat_max_msgs+1];

	for( i = 0; i < cat_max_msgs+1; i++ )
		freerec[i] = true;
	
	for( num = 0; ; num = next_read) {
		if( false == dat__read( f, &info, num )) 
			break;
		if( info.number >= 1 && info.number < cat_max_msgs+1 )
			freerec[info.number] = false;
	}
	
	for( i = 1; i < cat_max_msgs+1; i++ )
		if( freerec[i] == true )
			return i;

	return -1;
}

/* -----------------------------------
 * Hngt einen Eintrag an die Msginfo
 * an
 * ----------------------------------- */
 
static Boolean
dat__append( FILE *f, ShortInfo *info) 
{
	int count = dat__length( f);
	
	return (   count != -1 
	        && !fseek(f, 0, SEEK_END )
	        && 1 == fwrite( info, sizeof( ShortInfo), 1, f) ) ?
	        true : false;
}

/* ----------------------------------
 * Erzeugt eine neue msginfo.dat
 * ---------------------------------- */

static Boolean
dat__create( FILE *f ) 
{
	rewind(f);
	return 	fwrite( &magic, sizeof( magic), 1 , f) == 1 ? true : false;
}

/* ----------------------------------
 * ffnet die msginfo.dat. String-
 * lngen sind hier kein Problem mehr
 * ---------------------------------- */

static FILE *
dat__open( CatInfo *inf, const char *openmode)
{
	Path path;
	
	strcpy( path, inf->message_path);
	strcat( path, "MSGINFO.DAT");
	
	return fopen( path, openmode);
}

/* -----------------------------------
 * schliet die msginfo.dat
 * ----------------------------------- */
 
static int
dat__close( FILE *f)
{
	return fclose(f);
}

/* ==================================
 * Einige Funktionen um CatInfo zu 
 * fllen
 * ================================== */
 
/* ----------------------------------
 * Findet den Pfad von CAT.INF, oder
 * auch nicht ;-)
 * ---------------------------------- */
 
static Boolean
init__catinf( CatInfo *inf )
{
	char *env = getenv( "CATMAIL" );
	struct stat dummy;
	if( !env ) 
		env = "CAT.INF";
		
	strncpy( inf->inf_path, env, sizeof inf->inf_path)
		[ sizeof inf->inf_path - 1 ] = 0;
	
	return !stat( inf->inf_path, &dummy) ? true: false;
}

/* -----------------------------------
 * Sucht den Messageordner und legt
 * dort ein msginfo.dat an, falls das
 * noch nicht existiert
 * ----------------------------------- */

static Boolean
init__msgpath( CatInfo *inf) 
{
	FILE *f = fopen( inf->inf_path , "r");
	Path buffer;
	char *p;
	int  line = 8;
	Boolean ret;

	if( ! f ) 
		return false;
		
	while( line -- )
		if(!fgets( buffer, (int)sizeof(buffer), f))
			return false;
	
	p = strchr(buffer,'\n');
	if( p ) 
		*p = 0;
	strncpy( inf->message_path, buffer, sizeof inf->message_path)
		[ sizeof inf->message_path - 1 ] = 0;
	
	if( strlen( buffer ) + 12 >= sizeof( buffer ) )
		return false;
		
	fclose(f);
	
	f = dat__open( inf, "rb");
	ret = true;
	if( ! f) {
		f = dat__open( inf, "wb");
		if( !f )
			return false;
		ret = dat__create( f);
		fflush(f);
	}
	if( -1 == dat__length(f) )
		ret = false;
	dat__close(f);
	
	if( ret == true ) { /* wwc 24.12.93 -> */
		f = dat__open( inf, "rb");
		if( ! f )
			ret = false;
		else {
			inf->msg_number = dat__firstfree(f);
			if( inf->msg_number == -1 )
				ret = false;    
			dat__close(f);
		}
	}                   /* <- wwc 24.12.93 */
	
	return ret;
}

/* ---------------------------------
 * Die init Prozedur
 * --------------------------------- */
 
static Boolean
init( CatInfo *inf) 
{
	if( init__catinf( inf ) == false ) {
		fprintf(stderr,
			"%s nicht gefunden. Bitte catmail -h aufrufen.\n\r",
			inf->inf_path);
		return false;
	}
	if( init__msgpath( inf ) == false ) {
		fprintf(stderr,
			"%s hat nicht das erwartete Format.\n\r",
			inf->inf_path);
		return false;
	}		
	return true;
}
	
/* =================================
 * Funktion um den Header (.HDR)
 * zu erzeugen.
 * ================================= */
 
static Boolean
hdr__create( const char *filename, 
            const char *adresse, 
            const char *gruppe,
            const char *subject,
            struct tm *tm )
{
	FILE *h = fopen( filename, "w");
	if( h ) {
		fprintf(h, "%04d%02d%02d%02d%02d\n"
		           "%s\n"
	  	         "%s\n"
	    	       "%s\n"
	      	     "\n"
	        	   "%s\n"
	          	 "\n",
			(tm->tm_year % 100) + 1900,
			tm->tm_mon + 1,
			tm->tm_mday,
			tm->tm_hour,
			tm->tm_min,
			adresse,
			gruppe,
			gruppe,
			subject);
		fclose(h);
		return true;
 	}
 	return false;
}

/* ==================================
 * Funktion um den Messagetext (.TXT)
 * zu erzeugen.
 * ================================== */

static Boolean
txt__create( const char *filename, FILE *input , 
             unsigned long *plength)
{
	FILE *t = fopen( filename, "w");
	int c;
	struct stat st;
	if( t ) {
		if( input == stdin && isatty(fileno(stdin)))
			fprintf(stderr,
				"Geben Sie nun den Nachrichtentext ein."
				" Ende mit CONTROL-Z am Zeilenanfang\n\r");
		while( EOF != ( c = fgetc(input)) )
			fputc(c,t);
		fclose(t);
		stat( (char *)filename, &st);
		*plength = st.st_size;
		return true;
 	}
 	return false;
}

/* ---------------------------------
 * wandelt _ in space um
 * (fr adresse) 
 * --------------------------------- */

void 
_tospace( char *q)
{
	while( *q) {
		if( *q == '_' )
			*q = ' ';
		q++;
	}
}

/* ---------------------------------
 * catmain
 * --------------------------------- */

int
main( int argc, char **argv)
{
	char adresse[256];
	char maus_gruppe[64];
	char subject[64];
	static Path input_file; /* save stack */
	static Path output_file;
	int i;
	Boolean result;
	CatInfo catinfo;
	ShortInfo shortinfo;
	FILE *in, *mi;
	time_t t_t;
	struct tm *tm;
	
	*adresse = *maus_gruppe = *subject = *input_file = 0;
	
	/* --------------------------------------------------------------
	    Auswertung der Argumente */
	
	for( i = 1; i < argc; i++ ) {
		if( !strcmp("-a",argv[i]) && i < argc-1 )
			strncpy( maus_gruppe, argv[++i], sizeof maus_gruppe)
				[sizeof maus_gruppe - 1 ] = 0;
		else if( !strcmp("-i",argv[i]) && i < argc-1 )
			strncpy( input_file, argv[++i], sizeof input_file)
				[sizeof input_file - 1 ] = 0;
		else if( !strcmp("-s",argv[i]) && i < argc-1 )
			strncpy( subject, argv[++i], sizeof subject)
				[sizeof subject - 1 ] = 0;
		else 
			strncpy( adresse, argv[i], sizeof adresse)
				[sizeof adresse - 1 ] = 0;
	}

	/* --------------------------------------------------------------
	    berprfung ;-) der Argumente */

	if( *adresse ) 
		_tospace(adresse);
		
	if( (!*maus_gruppe && !*adresse) || !subject ||
	    (*maus_gruppe && *adresse ) )
		syntax(); /* exits */
		
	/* --------------------------------------------------------------
	    Statusbericht nach der Auswertung */

	printf("CatMail:\n"); /* fr logfiles */
	if( *maus_gruppe ) 
		printf("ffentliche Mail in \"%s\"\n",maus_gruppe);
	if( *adresse ) 
		printf("Persnliche Mail an \"%s\"\n",adresse);
	if( *input_file ) 
		printf("Wird gelesen aus    \"%s\"\n",input_file);
	printf("Unter dem Betreff   \"%s\"\n",subject);
	
	/* --------------------------------------------------------------
	    Nach Cat suchen */

	if( init( &catinfo) == false) {
		fprintf(stderr,"Initialisierung fehlgeschlagen\n\r");
		exit(1);
	}
	
	/* --------------------------------------------------------------
	    msginfo.dat eintrag anfertigen (noch nicht schreiben!) */
	
	shortinfo.msg_type   = *adresse ? private : mail;
	shortinfo.number     = catinfo.msg_number;
	shortinfo.copy_count = 0;
	shortinfo.ref_num    = 0;
	shortinfo.send_msg   = true;            /* was sonst ;) */
	shortinfo.to_delete  = false;
	shortinfo.dist       = dNone;           /* Das kann man ja noch einbaun */
	shortinfo.no_ref_line= false;
	shortinfo.c_date     = Tgetdate();
	shortinfo.c_time     = Tgettime();
	t_t = time(0);
	tm  = localtime(&t_t);                  /* Zeit synchron aufnehmen */

	/* --------------------------------------------------------------
	    Jetzt das Textfile schreiben */
	
	if( *input_file ) {
		in = fopen( input_file,"r");
		if( !in ) {
			perror( input_file);
			exit(1);
		}
	} else
		in = stdin;
		
	sprintf(output_file,"%sMSG%03d.TXT", 
		catinfo.message_path,catinfo.msg_number);
	result = txt__create( output_file, in, &shortinfo.size);
	
	if( in != stdin )
		fclose(in);
	
	if( result == false) {
		fprintf(stderr,"Kann %s nicht schreiben\n",output_file);
		exit(1);
	}
	
	/* --------------------------------------------------------------
	    Jetzt den Header schreiben */
	
	sprintf(output_file,"%sMSG%03d.HDR", 
		catinfo.message_path,catinfo.msg_number);
	if( hdr__create( output_file, adresse, maus_gruppe, subject, tm) 
	    == false) {
		fprintf(stderr,"Kann %s nicht schreiben\n",output_file);
		exit(1);
	}
	
	/* --------------------------------------------------------------
	    msginfo.dat updaten */
	    
	mi = dat__open(&catinfo, "r+b");
	if( !mi ) {
		perror("MSGINFO.DAT");
		exit(1);
	}
	
	if( dat__check( mi ) == false ) {
		fprintf(stderr,"MSGINFO.DAT: Unbekannte Cat-Version.\n");
		exit(1);
	}
	
	if( dat__append( mi, &shortinfo) == false ) {
		fprintf(stderr,"MSGINFO.DAT: Kann neue Mail nicht eintragen.\n");
		exit(1);
	}

	dat__close(mi);
	
	/* ------------------------------------------------------------------
	     Erfolg vermelden */
	     
	puts("Mail verschickt.");
	
	return 0;
}
	